@@ -239,6 +239,7 @@ inline void SPIRVMap<Attribute::AttrKind, SPIRVFuncParamAttrKind>::init() {
   add(Attribute::StructRet, FunctionParameterAttributeSret);
   add(Attribute::NoAlias, FunctionParameterAttributeNoAlias);
   add(Attribute::NoCapture, FunctionParameterAttributeNoCapture);
+  add(Attribute::ReadOnly, FunctionParameterAttributeNoWrite);
 }
 typedef SPIRVMap<Attribute::AttrKind, SPIRVFuncParamAttrKind>
     SPIRSPIRVFuncParamAttrMap;
@@ -362,6 +363,9 @@ const static char TranslateSPIRVMemFence[] = "__translate_spirv_memory_fence";
 } // namespace kSPIRVName
 
 namespace kSPIRVPostfix {
+const static char ToGlobal[] = "ToGlobal";
+const static char ToLocal[] = "ToLocal";
+const static char ToPrivate[] = "ToPrivate";
 const static char Sat[] = "sat";
 const static char Rtz[] = "rtz";
 const static char Rte[] = "rte";
@@ -402,6 +406,11 @@ const static char NumSIMD[] = "num_simd_work_items";
 const static char StallEnable[] = "stall_enable";
 const static char FmaxMhz[] = "scheduler_target_fmax_mhz";
 const static char LoopFuse[] = "loop_fuse";
+const static char PreferDSP[] = "prefer_dsp";
+const static char PropDSPPref[] = "propagate_dsp_preference";
+const static char InitiationInterval[] = "initiation_interval";
+const static char MaxConcurrency[] = "max_concurrency";
+const static char DisableLoopPipelining[] = "disable_loop_pipelining";
 } // namespace kSPIR2MD
 
 enum Spir2SamplerKind {
@@ -574,6 +583,7 @@ PointerType *getOrCreateOpaquePtrType(Module *M, const std::string &Name,
                                       unsigned AddrSpace = SPIRAS_Global);
 PointerType *getSamplerType(Module *M);
 PointerType *getPipeStorageType(Module *M);
+PointerType *getSPIRVOpaquePtrType(Module *M, Op OC);
 void getFunctionTypeParameterTypes(llvm::FunctionType *FT,
                                    std::vector<Type *> &ArgTys);
 Function *getOrCreateFunction(Module *M, Type *RetTy, ArrayRef<Type *> ArgTypes,
@@ -632,11 +642,15 @@ StringRef undecorateSPIRVFunction(StringRef S);
 /// and get the original name.
 bool isDecoratedSPIRVFunc(const Function *F, StringRef &UndecName);
 
+StringRef dePrefixSPIRVName(StringRef R, SmallVectorImpl<StringRef> &Postfix);
+
 /// Get a canonical function name for a SPIR-V op code.
 std::string getSPIRVFuncName(Op OC, StringRef PostFix = "");
 
 std::string getSPIRVFuncName(Op OC, const Type *PRetTy, bool IsSigned = false);
 
+std::string getSPIRVFuncName(SPIRVBuiltinVariableKind BVKind);
+
 /// Get a canonical function name for a SPIR-V extended instruction
 std::string getSPIRVExtFuncName(SPIRVExtInstSetKind Set, unsigned ExtOp,
                                 StringRef PostFix = "");
@@ -794,7 +808,7 @@ std::vector<Value *> getInt32(Module *M, const std::vector<int> &Value);
 ConstantInt *getSizet(Module *M, uint64_t Value);
 
 /// Get metadata operand as int.
-int getMDOperandAsInt(MDNode *N, unsigned I);
+int64_t getMDOperandAsInt(MDNode *N, unsigned I);
 
 /// Get metadata operand as string.
 std::string getMDOperandAsString(MDNode *N, unsigned I);
@@ -969,14 +983,29 @@ PointerType *getInt8PtrTy(PointerType *T);
 Value *castToInt8Ptr(Value *V, Instruction *Pos);
 
 template <> inline void SPIRVMap<std::string, Op, SPIRVOpaqueType>::init() {
-  add(kSPIRVTypeName::DeviceEvent, OpTypeDeviceEvent);
-  add(kSPIRVTypeName::Event, OpTypeEvent);
-  add(kSPIRVTypeName::Image, OpTypeImage);
-  add(kSPIRVTypeName::Pipe, OpTypePipe);
-  add(kSPIRVTypeName::Queue, OpTypeQueue);
-  add(kSPIRVTypeName::ReserveId, OpTypeReserveId);
-  add(kSPIRVTypeName::Sampler, OpTypeSampler);
-  add(kSPIRVTypeName::SampledImg, OpTypeSampledImage);
+#define _SPIRV_OP(x) add(#x, OpType##x);
+  _SPIRV_OP(DeviceEvent)
+  _SPIRV_OP(Event)
+  _SPIRV_OP(Image)
+  _SPIRV_OP(Pipe)
+  _SPIRV_OP(Queue)
+  _SPIRV_OP(ReserveId)
+  _SPIRV_OP(Sampler)
+  _SPIRV_OP(SampledImage)
+  // SPV_INTEL_device_side_avc_motion_estimation types
+  _SPIRV_OP(AvcMcePayloadINTEL)
+  _SPIRV_OP(AvcImePayloadINTEL)
+  _SPIRV_OP(AvcRefPayloadINTEL)
+  _SPIRV_OP(AvcSicPayloadINTEL)
+  _SPIRV_OP(AvcMceResultINTEL)
+  _SPIRV_OP(AvcImeResultINTEL)
+  _SPIRV_OP(AvcImeResultSingleReferenceStreamoutINTEL)
+  _SPIRV_OP(AvcImeResultDualReferenceStreamoutINTEL)
+  _SPIRV_OP(AvcImeSingleReferenceStreaminINTEL)
+  _SPIRV_OP(AvcImeDualReferenceStreaminINTEL)
+  _SPIRV_OP(AvcRefResultINTEL)
+  _SPIRV_OP(AvcSicResultINTEL)
+#undef _SPIRV_OP
 }
 
 // Check if the module contains llvm.loop.* metadata
@@ -988,6 +1017,43 @@ bool isSPIRVOCLExtInst(const CallInst *CI, OCLExtOpKind *ExtOp);
 
 // check LLVM Intrinsics type(s) for validity
 bool checkTypeForSPIRVExtendedInstLowering(IntrinsicInst *II, SPIRVModule *BM);
+
+/// Decode SPIR-V type name in the format spirv.{TypeName}._{Postfixes}
+/// where Postfixes are strings separated by underscores.
+/// \return TypeName.
+/// \param Strs contains the integers decoded from postfixes.
+std::string decodeSPIRVTypeName(StringRef Name,
+                                SmallVectorImpl<std::string> &Strs);
+
+// Copy attributes from function to call site.
+void setAttrByCalledFunc(CallInst *Call);
+bool isSPIRVBuiltinVariable(GlobalVariable *GV, SPIRVBuiltinVariableKind *Kind);
+// Transform builtin variable from GlobalVariable to builtin call.
+// e.g.
+// - GlobalInvolcationId[x] -> _Z33__spirv_BuiltInGlobalInvocationIdi(x)
+// - WorkDim -> _Z22__spirv_BuiltInWorkDimv()
+bool lowerBuiltinVariableToCall(GlobalVariable *GV,
+                                SPIRVBuiltinVariableKind Kind);
+// Transform all builtin variables into calls
+bool lowerBuiltinVariablesToCalls(Module *M);
+
+/// \brief Post-process OpenCL or SPIRV builtin function returning struct type.
+///
+/// Some builtin functions are translated to SPIR-V instructions with
+/// struct type result, e.g. NDRange creation functions. Such functions
+/// need to be post-processed to return the struct through sret argument.
+bool postProcessBuiltinReturningStruct(Function *F);
+
+/// \brief Post-process OpenCL or SPIRV builtin function having array argument.
+///
+/// These functions are translated to functions with array type argument
+/// first, then post-processed to have pointer arguments.
+bool postProcessBuiltinWithArrayArguments(Function *F, StringRef DemangledName);
+
+bool postProcessBuiltinsReturningStruct(Module *M, bool IsCpp = false);
+
+bool postProcessBuiltinsWithArrayArguments(Module *M, bool IsCpp = false);
+
 } // namespace SPIRV
 
 #endif // SPIRV_SPIRVINTERNAL_H
