@@ -1,12 +1,14 @@
 #ifndef SPIRV_DEBUG_H
 #define SPIRV_DEBUG_H
 #include "SPIRVUtil.h"
+#include "spirv/unified1/spirv.hpp"
 #include "llvm/BinaryFormat/Dwarf.h"
 
 namespace SPIRVDebug {
 
 const unsigned int DebugInfoVersion = 0x00010000;
 static const std::string ProducerPrefix = {"Debug info producer: "};
+static const std::string ChecksumKindPrefx = {"//__CSK_"};
 
 // clang-format off
 
@@ -47,7 +49,8 @@ enum Instruction {
   MacroUndef                    = 33,
   ImportedEntity                = 34,
   Source                        = 35,
-  InstCount                     = 36
+  ModuleINTEL                   = 36,
+  InstCount                     = 37
 };
 
 enum Flag {
@@ -769,11 +772,63 @@ enum {
 };
 }
 
+namespace ModuleINTEL {
+enum {
+  NameIdx         = 0,
+  SourceIdx       = 1,
+  LineIdx         = 2,
+  ParentIdx       = 3,
+  ConfigMacrosIdx = 4,
+  IncludePathIdx  = 5,
+  ApiNotesIdx     = 6,
+  IsDeclIdx       = 7,
+  OperandCount    = 8
+};
+}
+
 } // namespace Operand
 } // namespace SPIRVDebug
 
 using namespace llvm;
 
+inline spv::SourceLanguage convertDWARFSourceLangToSPIRV(dwarf::SourceLanguage DwarfLang) {
+  switch (DwarfLang) {
+  // When updating this function, make sure to also
+  // update convertSPIRVSourceLangToDWARF()
+
+  // LLVM does not yet define DW_LANG_C_plus_plus_17
+  // case dwarf::SourceLanguage::DW_LANG_C_plus_plus_17:
+  case dwarf::SourceLanguage::DW_LANG_C_plus_plus_14:
+  case dwarf::SourceLanguage::DW_LANG_C_plus_plus:
+    return spv::SourceLanguage::SourceLanguageCPP_for_OpenCL;
+  case dwarf::SourceLanguage::DW_LANG_C99:
+  case dwarf::SourceLanguage::DW_LANG_OpenCL:
+    return spv::SourceLanguage::SourceLanguageOpenCL_C;
+  default:
+    return spv::SourceLanguage::SourceLanguageUnknown;
+  }
+}
+
+inline dwarf::SourceLanguage convertSPIRVSourceLangToDWARF(unsigned SourceLang) {
+  switch (SourceLang) {
+  // When updating this function, make sure to also
+  // update convertDWARFSourceLangToSPIRV()
+  case spv::SourceLanguage::SourceLanguageOpenCL_CPP:
+    return dwarf::SourceLanguage::DW_LANG_C_plus_plus_14;
+  case spv::SourceLanguage::SourceLanguageCPP_for_OpenCL:
+    // LLVM does not yet define DW_LANG_C_plus_plus_17
+    // SourceLang = dwarf::SourceLanguage::DW_LANG_C_plus_plus_17;
+    return dwarf::SourceLanguage::DW_LANG_C_plus_plus_14;
+  case spv::SourceLanguage::SourceLanguageOpenCL_C:
+  case spv::SourceLanguage::SourceLanguageESSL:
+  case spv::SourceLanguage::SourceLanguageGLSL:
+  case spv::SourceLanguage::SourceLanguageHLSL:
+  case spv::SourceLanguage::SourceLanguageUnknown:
+  default:
+    return dwarf::DW_LANG_OpenCL;
+  }
+}
+
 namespace SPIRV {
 typedef SPIRVMap<dwarf::TypeKind, SPIRVDebug::EncodingTag> DbgEncodingMap;
 template <>
