@@ -41,7 +41,9 @@ config.test_exec_root = os.path.join(config.test_run_dir, 'test_output')
 
 llvm_config.use_default_substitutions()
 
-llvm_config.use_clang()
+# Explicitly set `use_installed` to alleviate downstream CI pipelines of
+# any additional environment setup for pre-installed Clang usage.
+llvm_config.use_clang(use_installed=True)
 
 config.substitutions.append(('%PATH%', config.environment['PATH']))
 
@@ -72,5 +74,5 @@ else:
     config.substitutions.append(('spirv-val', ':'))
 
 if using_spirv_tools:
-    new_ld_library_path = os.path.pathsep.join((config.spirv_tools_lib_dir, config.environment['LD_LIBRARY_PATH']))
-    config.environment['LD_LIBRARY_PATH'] = new_ld_library_path
+    llvm_config.with_system_environment('LD_LIBRARY_PATH')
+    llvm_config.with_environment('LD_LIBRARY_PATH', config.spirv_tools_lib_dir, append_path=True)
